@@ -309,17 +309,23 @@ void KDirWatchPrivate::inotifyEventReceived()
       if ( path.length() && isNoisyFile( cpath ) )
         continue;
 
+      // Is set to true if the new event is a directory, false otherwise. This prevents a stat call in clientsForFileOrDir
+      const bool isDir = (event->mask & (IN_ISDIR));
+
       // now we're in deep trouble of finding the
       // associated entries
       // for now, we suck and iterate
       for ( EntryMap::Iterator it = m_mapEntries.begin();
             it != m_mapEntries.end();  ) {
+
         Entry* e = &( *it );
         ++it;
         if ( e->wd == event->wd ) {
           const bool wasDirty = e->dirty;
           e->dirty = true;
 
+          const QString tpath = e->path + QLatin1Char('/') + path;
+
           //if (s_verboseDebug) {
           //  kDebug(7001) << "got event" << "0x"+QString::number(event->mask, 16) << "for" << e->path;
           //}
@@ -344,7 +350,6 @@ void KDirWatchPrivate::inotifyEventReceived()
             //e->wd = -1;
           }
           if ( event->mask & (IN_CREATE|IN_MOVED_TO) ) {
-            const QString tpath = e->path + QLatin1Char('/') + path;
             Entry* sub_entry = e->findSubEntry(tpath);
 
             if (s_verboseDebug) {
@@ -358,8 +363,7 @@ void KDirWatchPrivate::inotifyEventReceived()
               sub_entry->dirty = true;
               rescan_timer.start(0); // process this asap, to start watching that dir
             } else if (e->isDir && !e->m_clients.empty()) {
-              bool isDir = false;
-              const QList<Client *> clients = e->clientsForFileOrDir(tpath, &isDir);
+              const QList<Client *> clients = e->inotifyClientsForFileOrDir(isDir);
               Q_FOREACH(Client *client, clients) {
                 // See discussion in addEntry for why we don't addEntry for individual
                 // files in WatchFiles mode with inotify.
@@ -379,7 +383,6 @@ void KDirWatchPrivate::inotifyEventReceived()
             }
           }
           if (event->mask & (IN_DELETE|IN_MOVED_FROM)) {
-            const QString tpath = e->path + QLatin1Char('/') + path;
             if (s_verboseDebug) {
               kDebug(7001) << "-->got DELETE signal for" << tpath;
             }
@@ -389,14 +392,7 @@ void KDirWatchPrivate::inotifyEventReceived()
               // watched file as it would have its own watch descriptor, so
               // no addEntry/ removeEntry bookkeeping should be required.  Emit
               // the event immediately if any clients are interested.
-              KDE_struct_stat stat_buf;
-              // Unlike clientsForFileOrDir, the stat can fail here (item deleted),
-              // so in that case we'll just take both kinds of clients and emit Deleted.
-              KDirWatch::WatchModes flag = KDirWatch::WatchSubDirs | KDirWatch::WatchFiles;
-              if (KDE::stat(tpath, &stat_buf) == 0) {
-                bool isDir = S_ISDIR(stat_buf.st_mode);
-                flag = isDir ? KDirWatch::WatchSubDirs : KDirWatch::WatchFiles;
-              }
+              KDirWatch::WatchModes flag = isDir ? KDirWatch::WatchSubDirs : KDirWatch::WatchFiles;
               int counter = 0;
               Q_FOREACH(client, e->m_clients) { // krazy:exclude=foreach
                   if (client->m_watchModes & flag) {
@@ -410,7 +406,6 @@ void KDirWatchPrivate::inotifyEventReceived()
           }
           if (event->mask & (IN_MODIFY|IN_ATTRIB)) {
             if ((e->isDir) && (!e->m_clients.empty())) {
-              const QString tpath = e->path + QLatin1Char('/') + path;
               if (s_verboseDebug) {
                 kDebug(7001) << "-->got MODIFY signal for" << (tpath);
               }
@@ -549,6 +544,20 @@ QList<KDirWatchPrivate::Client *> KDirWatchPrivate::Entry::clientsForFileOrDir(c
   return ret;
 }
 
+// inotify specific function that doesn't call KDE::stat to figure out if we have a file or folder.
+// isDir is determined through inotify's "IN_ISDIR" flag in KDirWatchPrivate::inotifyEventReceived
+QList<KDirWatchPrivate::Client *> KDirWatchPrivate::Entry::inotifyClientsForFileOrDir(bool isDir) const
+{
+  QList<Client *> ret;
+  const KDirWatch::WatchModes flag = isDir ? KDirWatch::WatchSubDirs : KDirWatch::WatchFiles;
+  Q_FOREACH(Client *client, this->m_clients) {
+    if (client->m_watchModes & flag) {
+      ret.append(client);
+    }
+  }
+  return ret;
+}
+
 QDebug operator<<(QDebug debug, const KDirWatchPrivate::Entry &entry)
 {
   debug.nospace() << "[ Entry for " << entry.path << ", " << (entry.isDir ? "dir" : "file");
@@ -758,7 +767,8 @@ void KDirWatchPrivate::addEntry(KDirWatch* instance, const QString& _path,
   if (path.isEmpty()
 #ifndef Q_WS_WIN
      || path == QLatin1String("/dev")
-     || (path.startsWith(QLatin1String("/dev/")) && !path.startsWith(QLatin1String("/dev/.")))
+     || (path.startsWith(QLatin1String("/dev/")) && !path.startsWith(QLatin1String("/dev/."))
+	&& !path.startsWith(QLatin1String("/dev/shm")))
 #endif
   )
     return; // Don't even go there.
@@ -982,7 +992,7 @@ void KDirWatchPrivate::removeWatch(Entry* e)
 #endif
 }
 
-void KDirWatchPrivate::removeEntry(KDirWatch* instance,
+bool KDirWatchPrivate::removeEntry(KDirWatch* instance,
                                    const QString& _path,
                                    Entry* sub_entry)
 {
@@ -991,11 +1001,11 @@ void KDirWatchPrivate::removeEntry(KDirWatch* instance,
   }
   Entry* e = entry(_path);
   if (!e) {
-    kWarning(7001) << "doesn't know" << _path;
-    return;
+    return false;
   }
 
   removeEntry(instance, e, sub_entry);
+  return true;
 }
 
 void KDirWatchPrivate::removeEntry(KDirWatch* instance,
@@ -1808,12 +1818,20 @@ QDateTime KDirWatch::ctime( const QString &_path ) const
 
 void KDirWatch::removeDir( const QString& _path )
 {
-  if (d) d->removeEntry(this, _path, 0);
+    if (d) {
+        if (!d->removeEntry(this, _path, 0)) {
+            kWarning() << "doesn't know" << _path;
+        }
+    }
 }
 
 void KDirWatch::removeFile( const QString& _path )
 {
-  if (d) d->removeEntry(this, _path, 0);
+    if (d) {
+        if (!d->removeEntry(this, _path, 0)) {
+            kWarning() << "doesn't know" << _path;
+        }
+    }
 }
 
 bool KDirWatch::stopDirScan( const QString& _path )
