@@ -27,47 +27,43 @@
 #include <kglobal.h>
 #include <QtCore/QMap>
 
-class KStandardDirsPrivate;
 class KConfig;
 
 /**
  * @short Site-independent access to standard KDE directories.
  * @author Stephan Kulow <coolo@kde.org> and Sirtaj Singh Kang <taj@kde.org>
  *
- * This is one of the most central classes in kdelibs as
- * it provides a basic service: It knows where the files
- * reside on the user's hard disk. And it's meant to be the
- * only one that knows -- to make the real location as
- * transparent as possible to both the user and the applications.
+ * This is one of the most central classes in kdelibs: It knows where KDE-related files
+ * reside on the user's hard disk. It's meant to be the only one that knows --
+ * so applications and the end user don't have to.
  *
- * To this end it insulates the application from all information
- * and applications always refer to a file with a resource type
- * (e.g. icon) and a filename (e.g. <tt>khexdit.xpm</tt>). In an ideal world
- * the application would make no assumption where this file is and
- * leave it up to KStandardDirs::findResource("apps", "Home.desktop")
- * to apply this knowledge to return <tt>/opt/kde/share/applnk/Home.desktop</tt>
+ * Applications should always refer to a file with a resource type.
+ * The application should leave it up to e.g.
+ * KStandardDirs::findResource("apps", "Home.desktop")
+ * to return the desired path <tt>/opt/kde/share/applnk/Home.desktop</tt>
  * or ::locate("data", "kgame/background.jpg") to return
  * <tt>/opt/kde/share/apps/kgame/background.jpg</tt>
  *
- * The main idea behind KStandardDirs is that there are several
- * toplevel prefixes below which the files lie. One of these prefixes is
- * the one where the user installed kdelibs, one is where the
- * application was installed, and one is <tt>$HOME/.kde</tt>, but there
- * may be even more. Under these prefixes there are several well
- * defined suffixes where specific resource types are to be found.
+ * There are several toplevel prefixes under which files can be located.
+ * One of them is the kdelibs install location, one is the application
+ * install location, and one is <tt>$KDEHOME</tt>.
+ * Under these toplevel prefixes there are several well-defined suffixes
+ * where specific resource types can be found.
  * For example, for the resource type @c "html" the suffixes could be
  * @c share/doc/HTML and @c share/doc/kde/HTML.
- * So the search algorithm basically appends to each prefix each registered
- * suffix and tries to locate the file there.
- * To make the thing even more complex, it's also possible to register
+ * The search algorithm tries to locate the file under each prefix-suffix
+ * combination.
+ *
+ * It is also possible to register
  * absolute paths that KStandardDirs looks up after not finding anything
  * in the former steps. They can be useful if the user wants to provide
- * specific directories that aren't in his <tt>$HOME/.kde</tt> directory for,
- * for example, icons.
+ * specific directories that aren't in his <tt>$KDEHOME</tt> directory, for
+ * example for icons.
  *
- * <b>Standard resources that kdelibs allocates are:</b>\n
+ * <b>Standard resources that kdelibs allocates are:</b>
  *
  * @li @c apps - Applications menu (.desktop files).
+ * @li @c autostart - Autostart directories (both XDG and kde-specific)
  * @li @c cache - Cached information (e.g. favicons, web-pages)
  * @li @c cgi - CGIs to run from kdehelp.
  * @li @c config - Configuration files.
@@ -177,7 +173,18 @@ class KDECORE_EXPORT KStandardDirs
 public:
     /**
      * KStandardDirs' constructor. It just initializes the caches.
-     **/
+     * Note that you should normally not call this, but use KGlobal::dirs()
+     * instead, in order to reuse the same KStandardDirs object as much as possible.
+     *
+     * Creating other KStandardDirs instances can be useful in other threads.
+     *
+     * Thread safety note: using a shared KStandardDirs instance (such as KGlobal::dirs())
+     * in multiple threads is thread-safe if you only call the readonly "lookup" methods
+     * (findExe, resourceDirs, findDirs, findResourceDir, findAllResources, saveLocation,
+     * relativeLocation). The methods that modify the object (all those starting with "add",
+     * basically all non-const methods) are obviously not thread-safe; set things up
+     * before creating threads.
+     */
     KStandardDirs();
 
     enum SearchOption { NoSearchOptions = 0,
@@ -189,7 +196,7 @@ public:
     /**
      * KStandardDirs' destructor.
      */
-	virtual ~KStandardDirs();
+    virtual ~KStandardDirs();
 
     /**
      * Adds another search dir to front of the @p fsstnd list.
@@ -241,10 +248,12 @@ public:
      * otherwise after
      * @return true if successful, false otherwise.
      *
-     * @deprecated
+     * @deprecated, use addResourceType(type, 0, relativename, priority)
      */
+#ifndef KDE_NO_DEPRECATED
     KDE_DEPRECATED bool addResourceType( const char *type,
                                          const QString& relativename, bool priority = true );
+#endif
 
     /**
      * Adds suffixes for types.
@@ -307,7 +316,16 @@ public:
      * to findResource("lib", "libkdecore.la"). KStandardDirs will
      * then look into the subdir lib of all elements of all prefixes
      * ($KDEDIRS) for a file libkdecore.la and return the path to
-     * the first one it finds (e.g. /opt/kde/lib/libkdecore.la)
+     * the first one it finds (e.g. /opt/kde/lib/libkdecore.la).
+     * You can use the program kde4-config to list all resource types:
+     * @code
+     * $ kde4-config --types
+     * @endcode
+     *
+     * Example:
+     * @code
+     * QString iconfilename=KGlobal::dirs()->findResource("icon",QString("oxygen/22x22/apps/ktip.png"));
+     * @endcode
      *
      * @param type The type of the wanted resource
      * @param filename A relative filename of the resource.
@@ -354,10 +372,12 @@ public:
 
     /**
      * Tries to find all directories whose names consist of the
-     * specified type and a relative path. So would
-     * findDirs("apps", "Settings") return
-     * @li /opt/kde/share/applnk/Settings/
+     * specified type and a relative path. So
+     * findDirs("apps", "Settings") would return
      * @li /home/joe/.kde/share/applnk/Settings/
+     * @li /opt/kde/share/applnk/Settings/
+     *
+     * (from the most local to the most global)
      *
      * Note that it appends @c / to the end of the directories,
      * so you can use this right away as directory names.
@@ -401,6 +421,8 @@ public:
      * The function will look into all specified directories
      * and return all filenames in these directories.
      *
+     * The "most local" files are returned before the "more global" files.
+     *
      * @param type The type of resource to locate directories for.
      * @param filter Only accept filenames that fit to filter. The filter
      *        may consist of an optional directory and a QRegExp
@@ -424,7 +446,13 @@ public:
      * and return all filenames (full and relative paths) in
      * these directories.
      *
-     * @param type The type of resource to locate directories for.
+     * The "most local" files are returned before the "more global" files.
+     *
+     * @param type The type of resource to locate directories for. Can be icon,
+     *        lib, pixmap, .... To get a complete list, call
+     *        @code
+     *        kde4-config --types
+     *        @endcode
      * @param filter Only accept filenames that fit to filter. The filter
      *        may consist of an optional directory and a QRegExp
      *        wildcard expression. E.g. <tt>"images\*.jpg"</tt>.
@@ -463,7 +491,7 @@ public:
      *
      * @param appname The name of the executable file for which to search.
      *                if this contains a path separator, it will be resolved
-     *                according to the current working directory 
+     *                according to the current working directory
      *                (shell-like behaviour).
      * @param pathstr The path which will be searched. If this is
      *                null (default), the @c $PATH environment variable will
@@ -621,7 +649,9 @@ public:
      * @see locate()
      * @see locateLocal()
      */
+#ifndef KDE_NO_DEPRECATED
     static KDE_DEPRECATED QString kde_default(const char *type);
+#endif
 
     /**
      * @internal (for use by sycoca only)
@@ -764,9 +794,6 @@ private:
     class KStandardDirsPrivate;
     KStandardDirsPrivate* const d;
 
-    void applyDataRestrictions(const QString &) const;
-    void createSpecialResource(const char*);
-
     // Like their public counter parts but with an extra priority argument
     // If priority is true, the directory is added directly after
     // $KDEHOME/$XDG_DATA_HOME/$XDG_CONFIG_HOME
