@@ -1,5 +1,5 @@
 /* This file is part of the KDE libraries
-   Copyright (c) 2005-2007 David Jarvie <software@astrojar.org.uk>
+   Copyright (c) 2005-2007,2011-2012 David Jarvie <djarvie@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -17,92 +17,33 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include "ktimezonestest_p.h"
+
 #include <stdio.h>
 #include <qtest_kde.h>
 #include <QtCore/QDir>
 #include <QtCore/QDate>
 #include <QtDBus/QtDBus>
-#include "config.h" // for HAVE_TM_GMTOFF
+#include <config-date.h> // for HAVE_TM_GMTOFF
 #include "ksystemtimezone.h"
 #include "ktzfiletimezone.h"
 #include "ktimezonestest.moc"
-#include <kconfiggroup.h>
 
 QTEST_KDEMAIN_CORE(KTimeZonesTest)
 
+TimeZoneTestData s_testData;
+
 void KTimeZonesTest::initTestCase()
 {
-    cleanupTestCase();
-
-    mDataDir = QDir::homePath() + "/.kde-unit-test/ktimezonestest";
-    QVERIFY(QDir().mkpath(mDataDir));
-    writeZoneTab(false);
-    QDir dir(mDataDir);
-    QVERIFY(dir.mkdir("Africa"));
-    QFile::copy(QString::fromLatin1(KDESRCDIR) + QLatin1String("/Cairo"), mDataDir + QLatin1String("/Africa/Cairo"));
-    QVERIFY(dir.mkdir("America"));
-    QFile::copy(QString::fromLatin1(KDESRCDIR) + QLatin1String("/Los_Angeles"), mDataDir + QLatin1String("/America/Los_Angeles"));
-    QVERIFY(dir.mkdir("Asia"));
-    QFile::copy(QString::fromLatin1(KDESRCDIR) + QLatin1String("/Dili"), mDataDir + QLatin1String("/Asia/Dili"));
-    QVERIFY(dir.mkdir("Europe"));
-    QFile::copy(QString::fromLatin1(KDESRCDIR) + QLatin1String("/Berlin"), mDataDir + QLatin1String("/Europe/Berlin"));
-    QFile::copy(QString::fromLatin1(KDESRCDIR) + QLatin1String("/London"), mDataDir + QLatin1String("/Europe/London"));
-    QFile::copy(QString::fromLatin1(KDESRCDIR) + QLatin1String("/Paris"), mDataDir + QLatin1String("/Europe/Paris"));
-
-    KConfig config("ktimezonedrc");
-    KConfigGroup group(&config, "TimeZones");
-    group.writeEntry("ZoneinfoDir", mDataDir);
-    group.writeEntry("Zonetab", mDataDir + QString::fromLatin1("/zone.tab"));
-    group.writeEntry("LocalZone", QString::fromLatin1("Europe/Paris"));
-    config.sync();
+    s_testData.setupTimeZoneTest(); // see ktimezonestest_p.h
+    mDataDir = s_testData.dataDir();
 }
 
 void KTimeZonesTest::cleanupTestCase()
 {
-    removeDir(QLatin1String("ktimezonestest/Africa"));
-    removeDir(QLatin1String("ktimezonestest/America"));
-    removeDir(QLatin1String("ktimezonestest/Asia"));
-    removeDir(QLatin1String("ktimezonestest/Europe"));
-    removeDir(QLatin1String("ktimezonestest"));
-    removeDir(QLatin1String("share/config"));
-    QDir().rmpath(QDir::homePath() + "/.kde-unit-test/share");
+    s_testData.cleanupTimeZoneTest();
 }
 
-void KTimeZonesTest::writeZoneTab(bool testcase)
-{
-    QFile f;
-    f.setFileName(mDataDir + QLatin1String("/zone.tab"));
-    f.open(QIODevice::WriteOnly);
-    QTextStream fStream(&f);
-    if (testcase)
-        fStream << "DE	+5230+01322	Europe/Berlin\n"
-                   "EG	+3003+03115	Africa/Cairo\n"
-                   "FR	+4852+00220	Europe/Paris\n"
-                   "XX	-512830+0001845	Europe/London	Greater Britain\n"
-                   "TL	-0833+12535	Asia/Dili\n"
-                   "US	+340308-1181434	America/Los_Angeles	Pacific Time\n";
-    else
-        fStream << "EG	+3003+03115	Africa/Cairo\n"
-                   "FR	+4852+00220	Europe/Paris\n"
-                   "GB	+512830-0001845	Europe/London	Great Britain\n"
-                   "US	+340308-1181434	America/Los_Angeles	Pacific Time\n";
-    f.close();
-}
-
-void KTimeZonesTest::removeDir(const QString &subdir)
-{
-    QDir local = QDir::homePath() + QLatin1String("/.kde-unit-test/") + subdir;
-    foreach(const QString &file, local.entryList(QDir::Files))
-        if(!local.remove(file))
-            qWarning("%s: removing failed", qPrintable( file ));
-    QCOMPARE((int)local.entryList(QDir::Files).count(), 0);
-    local.cdUp();
-    QString subd = subdir;
-    subd.remove(QRegExp("^.*/"));
-    local.rmpath(subd);
-}
-
-
 ///////////////////
 // KTimeZones tests
 ///////////////////
@@ -166,6 +107,7 @@ void KTimeZonesTest::utc()
     QCOMPARE(utc.name(), QString("UTC"));
     QCOMPARE(utc.offsetAtUtc(QDateTime(QDate(2005,1,1), QTime(), Qt::LocalTime)), 0);
     QCOMPARE(utc.offsetAtUtc(QDateTime(QDate(2005,1,1), QTime(), Qt::UTC)), 0);
+    QCOMPARE(utc.offsetAtUtc(QDateTime(QDate(2005,2,1), QTime(), Qt::UTC)), 0);
     QCOMPARE(utc.offsetAtUtc(QDateTime(QDate(2005,7,1), QTime(), Qt::UTC)), 0);
 }
 
@@ -178,7 +120,7 @@ void KTimeZonesTest::local()
     KTimeZone local = KSystemTimeZones::local();
     QVERIFY(local.isValid());
     QCOMPARE(local.name(), QString::fromLatin1("Europe/Paris"));
-    QCOMPARE(local.type(), QByteArray("KSystemTimeZone"));
+    QCOMPARE(local.type(), QByteArray("KTzfileTimeZone"));
 }
 
 void KTimeZonesTest::zone()
@@ -210,7 +152,7 @@ void KTimeZonesTest::zonetabChange()
     QObject::connect(&timer, SIGNAL(timeout()), &loop, SLOT(quit()));
     QSignalSpy timeoutSpy(&timer, SIGNAL(timeout()));
 
-    QCOMPARE(KSystemTimeZones::zones().count(), 4);
+    QCOMPARE(KSystemTimeZones::zones().count(), 5);
     KTimeZone london = KSystemTimeZones::zone("Europe/London");
     QVERIFY(london.isValid());
     QCOMPARE(london.countryCode(), QString("GB"));
@@ -226,15 +168,15 @@ void KTimeZonesTest::zonetabChange()
 
     // Check that 'london' is automatically updated with the new zone.tab
     // contents, and that the new zones are added to KSystemTimeZones.
-    writeZoneTab(true);
+    s_testData.writeZoneTab(true);
     QDBusMessage message = QDBusMessage::createSignal("/Daemon", "org.kde.KTimeZoned", "zonetabChanged");
     QList<QVariant> args;
-    args += mDataDir + QLatin1String("/zone.tab");
+    args += QString(mDataDir + QLatin1String("/zone.tab"));
     message.setArguments(args);
     QDBusConnection::sessionBus().send(message);
     timer.start(1000);
     loop.exec();
-    QCOMPARE(KSystemTimeZones::zones().count(), 6);
+    QCOMPARE(KSystemTimeZones::zones().count(), 7);
     QVERIFY(london.isValid());
     QCOMPARE(london.countryCode(), QString("XX"));
     QCOMPARE(london.latitude(), -float(51*3600 + 28*60 + 30)/3600.0f);
@@ -245,16 +187,17 @@ void KTimeZonesTest::zonetabChange()
     QVERIFY(KSystemTimeZones::zone("Europe/Paris").isValid());
     QVERIFY(KSystemTimeZones::zone("Europe/London").isValid());
     QVERIFY(KSystemTimeZones::zone("Africa/Cairo").isValid());
+    QVERIFY(KSystemTimeZones::zone("Africa/Johannesburg").isValid());
     QVERIFY(KSystemTimeZones::zone("Asia/Dili").isValid());
     QVERIFY(KSystemTimeZones::zone("America/Los_Angeles").isValid());
 
     // Check that 'london' is automatically updated with the new zone.tab
     // contents, and that the removed zones are deleted from KSystemTimeZones.
-    writeZoneTab(false);
+    s_testData.writeZoneTab(false);
     QDBusConnection::sessionBus().send(message);
     timer.start(1000);
     loop.exec();
-    QCOMPARE(KSystemTimeZones::zones().count(), 4);
+    QCOMPARE(KSystemTimeZones::zones().count(), 5);
     QVERIFY(london.isValid());
     QCOMPARE(london.countryCode(), QString("GB"));
     QCOMPARE(london.latitude(), float(51*3600 + 28*60 + 30)/3600.0f);
@@ -276,7 +219,7 @@ void KTimeZonesTest::zonetabChange()
 
 void KTimeZonesTest::currentOffset()
 {
-    QString tzfile = ":" + mDataDir + "/Europe/Paris";
+    QString tzfile = ':' + mDataDir + "/Europe/Paris";
     const char *originalZone = ::getenv("TZ");   // save the original local time zone
     ::setenv("TZ", tzfile.toLatin1().data(), 1);
     ::tzset();
@@ -306,9 +249,9 @@ void KTimeZonesTest::offsetAtUtc()
 {
     // Find some offsets for Europe/London.
     KTimeZone losAngeles = KSystemTimeZones::zone("America/Los_Angeles");
-    QVERIFY( losAngeles.isValid() );
+    QVERIFY(losAngeles.isValid());
     KTimeZone london = KSystemTimeZones::zone("Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     QDateTime winter(QDate(2005,1,1), QTime(0,0,0), Qt::UTC);
     QDateTime summer(QDate(2005,6,1), QTime(0,0,0), Qt::UTC);
     QCOMPARE(london.offsetAtUtc(winter), 0);
@@ -319,36 +262,60 @@ void KTimeZonesTest::offsetAtUtc()
 
 void KTimeZonesTest::offsetAtZoneTime()
 {
-    QDateTime aGmt(QDate(2005,3,27), QTime(0,30,0), Qt::LocalTime);
-    QDateTime aInvalid(QDate(2005,3,27), QTime(1,30,0), Qt::LocalTime);
-    QDateTime aBst(QDate(2005,3,27), QTime(2,30,0), Qt::LocalTime);
-    QDateTime bBst(QDate(2005,10,30), QTime(0,30,0), Qt::LocalTime);
-    QDateTime bBstBeforeGmt(QDate(2005,10,30), QTime(1,30,0), Qt::LocalTime);
-    QDateTime bGmt(QDate(2005,10,30), QTime(2,30,0), Qt::LocalTime);
+    QDateTime Gmt1(QDate(2005,3,27), QTime(0,30,0), Qt::LocalTime);
+    QDateTime GmtToBst1(QDate(2005,3,27), QTime(0,59,59), Qt::LocalTime);
+    QDateTime GmtToBst2(QDate(2005,3,27), QTime(1,0,0), Qt::LocalTime);
+    QDateTime GmtToBst3(QDate(2005,3,27), QTime(1,59,59), Qt::LocalTime);
+    QDateTime GmtToBst4(QDate(2005,3,27), QTime(2,0,0), Qt::LocalTime);
+    QDateTime Bst1(QDate(2005,4,27), QTime(2,30,0), Qt::LocalTime);
+    QDateTime Bst2(QDate(2005,10,30), QTime(0,30,0), Qt::LocalTime);
+    QDateTime BstToGmt1(QDate(2005,10,30), QTime(0,59,59), Qt::LocalTime);
+    QDateTime BstToGmt2(QDate(2005,10,30), QTime(1,0,0), Qt::LocalTime);
+    QDateTime BstToGmt3(QDate(2005,10,30), QTime(1,59,59), Qt::LocalTime);
+    QDateTime BstToGmt4(QDate(2005,10,30), QTime(2,0,0), Qt::LocalTime);
+    QDateTime Gmt2(QDate(2005,10,30), QTime(2,30,0), Qt::LocalTime);
     KTimeZone london = KSystemTimeZones::zone("Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     int offset2;
-    QCOMPARE(london.offsetAtZoneTime(aGmt, &offset2), 0);
+    QCOMPARE(london.offsetAtZoneTime(Gmt1, &offset2), 0);
     QCOMPARE(offset2, 0);
-    QCOMPARE(london.offsetAtZoneTime(aInvalid, &offset2), 3600);
-    QCOMPARE(offset2, 3600);
-    QCOMPARE(london.offsetAtZoneTime(aBst, &offset2), 3600);
-    QCOMPARE(offset2, 3600);
-    QCOMPARE(london.offsetAtZoneTime(bBst, &offset2), 3600);
-    QCOMPARE(offset2, 3600);
-    QCOMPARE(london.offsetAtZoneTime(bBstBeforeGmt, &offset2), 3600);
+    QCOMPARE(london.offsetAtZoneTime(GmtToBst1, &offset2), 0);
     QCOMPARE(offset2, 0);
-    QCOMPARE(london.offsetAtZoneTime(bGmt, &offset2), 0);
+    QCOMPARE(london.offsetAtZoneTime(GmtToBst2, &offset2), KTimeZone::InvalidOffset);
+    QCOMPARE(offset2, KTimeZone::InvalidOffset);
+    QCOMPARE(london.offsetAtZoneTime(GmtToBst3, &offset2), KTimeZone::InvalidOffset);
+    QCOMPARE(offset2, KTimeZone::InvalidOffset);
+    QCOMPARE(london.offsetAtZoneTime(GmtToBst4, &offset2), 3600);
+    QCOMPARE(offset2, 3600);
+    QCOMPARE(london.offsetAtZoneTime(Bst1, &offset2), 3600);
+    QCOMPARE(offset2, 3600);
+    QCOMPARE(london.offsetAtZoneTime(Bst2, &offset2), 3600);
+    QCOMPARE(offset2, 3600);
+    QCOMPARE(london.offsetAtZoneTime(BstToGmt1, &offset2), 3600);
+    QCOMPARE(offset2, 3600);
+    QCOMPARE(london.offsetAtZoneTime(BstToGmt2, &offset2), 3600);
     QCOMPARE(offset2, 0);
+    QCOMPARE(london.offsetAtZoneTime(BstToGmt3, &offset2), 3600);
+    QCOMPARE(offset2, 0);
+    QCOMPARE(london.offsetAtZoneTime(BstToGmt4, &offset2), 0);
+    QCOMPARE(offset2, 0);
+    QCOMPARE(london.offsetAtZoneTime(Gmt2, &offset2), 0);
+    QCOMPARE(offset2, 0);
+
+    KTimeZone johannesburg = KSystemTimeZones::zone("Africa/Johannesburg");
+    QVERIFY(johannesburg.isValid());
+    QDateTime recent(QDate(2013,5,10), QTime(13,0,0), Qt::LocalTime);
+    QCOMPARE(johannesburg.offsetAtZoneTime(recent, &offset2), 7200);
+    QCOMPARE(offset2, 7200);
 }
 
 void KTimeZonesTest::abbreviation()
 {
     // Fetch time zone abbreviations
     KTimeZone losAngeles = KSystemTimeZones::zone("America/Los_Angeles");
-    QVERIFY( losAngeles.isValid() );
+    QVERIFY(losAngeles.isValid());
     KTimeZone london = KSystemTimeZones::zone("Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     QDateTime winter(QDate(2005,1,1), QTime(0,0,0), Qt::UTC);
     QDateTime summer(QDate(2005,6,1), QTime(0,0,0), Qt::UTC);
     QString sResult = london.abbreviation(winter);
@@ -374,9 +341,9 @@ void KTimeZonesTest::toUtc()
 {
     // Convert to UTC.
     KTimeZone losAngeles = KSystemTimeZones::zone("America/Los_Angeles");
-    QVERIFY( losAngeles.isValid() );
+    QVERIFY(losAngeles.isValid());
     KTimeZone london = KSystemTimeZones::zone("Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     QDateTime winter(QDate(2005,1,1), QTime(0,0,0), Qt::UTC);
     QDateTime summer(QDate(2005,6,1), QTime(0,0,0), Qt::UTC);
     QDateTime winterLocal = winter;
@@ -393,9 +360,9 @@ void KTimeZonesTest::toZoneTime()
 {
     // Convert from UTC.
     KTimeZone losAngeles = KSystemTimeZones::zone("America/Los_Angeles");
-    QVERIFY( losAngeles.isValid() );
+    QVERIFY(losAngeles.isValid());
     KTimeZone london = KSystemTimeZones::zone("Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     QDateTime winter(QDate(2005,1,1), QTime(0,0,0), Qt::UTC);
     QDateTime summer(QDate(2005,6,1), QTime(0,0,0), Qt::UTC);
     QDateTime winterLocal = winter;
@@ -432,9 +399,9 @@ void KTimeZonesTest::convert()
 {
     // Try time zone conversions.
     KTimeZone losAngeles = KSystemTimeZones::zone("America/Los_Angeles");
-    QVERIFY( losAngeles.isValid() );
+    QVERIFY(losAngeles.isValid());
     KTimeZone london = KSystemTimeZones::zone("Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     QDateTime bstBeforePdt(QDate(2005,3,28), QTime(0,0,0), Qt::LocalTime);
     QDateTime bstAfterPdt(QDate(2005,5,1), QTime(0,0,0), Qt::LocalTime);
     QDateTime gmtBeforePst(QDate(2005,10,30), QTime(4,0,0), Qt::LocalTime);
@@ -467,6 +434,48 @@ void KTimeZonesTest::tzfile()
     tzcairo = new KTzfileTimeZone(&tzsource, "Africa/Cairo");
     QCOMPARE(tzcairo->offsetAtUtc(winter), 7200);
     delete tzcairo;
+    KTimeZone *johannesburg = new KTzfileTimeZone(&tzsource, "Africa/Johannesburg");
+    delete johannesburg;
+    johannesburg = new KTzfileTimeZone(&tzsource, "Africa/Johannesburg");
+    QCOMPARE(johannesburg->offsetAtUtc(winter), 7200);
+    delete johannesburg;
+}
+
+void KTimeZonesTest::tzfileDstShifts()
+{
+    // Check time zone conversions against zdump output for zone
+    KTzfileTimeZoneSource tzsource(KSystemTimeZones::zoneinfoDir());
+    KTimeZone berlin = KTzfileTimeZone(&tzsource, "Europe/Berlin");
+    KTimeZone losAngeles = KTzfileTimeZone(&tzsource, "America/Los_Angeles");
+    KTimeZone tz = berlin;
+    for (int n = 0; n < 2; ++n, tz = losAngeles)
+    {
+        qDebug() << tz.name();
+        QVERIFY(tz.isValid());
+        QCOMPARE(tz.type(), QByteArray("KTzfileTimeZone"));
+        QFile file(QString::fromLatin1(KDESRCDIR) + tz.name().remove(QRegExp("^.+/")) + QLatin1String(".zdump"));
+        QVERIFY(file.open(QIODevice::ReadOnly));
+        QTextStream in(&file);
+        QString line;
+        while (!(line = in.readLine()).isNull())
+        {
+            QStringList parts = line.split(" ", QString::SkipEmptyParts);
+            QCOMPARE(parts[6], QString::fromLatin1("UTC"));
+            QCOMPARE(parts[7], QString::fromLatin1("="));
+            QCOMPARE(parts[14].left(6), QString::fromLatin1("isdst="));
+            QCOMPARE(parts[15].left(7), QString::fromLatin1("gmtoff="));
+            QDateTime utc = QDateTime::fromString(static_cast<QStringList>(parts.mid(1, 5)).join(" "));
+            utc.setTimeSpec(Qt::UTC);
+            QDateTime local = QDateTime::fromString(static_cast<QStringList>(parts.mid(8, 5)).join(" "));
+            local.setTimeSpec(Qt::LocalTime);
+            QCOMPARE(tz.toZoneTime(utc), local);
+            QCOMPARE(parts[13], QString::fromLatin1(tz.abbreviation(utc)));
+            bool dst = (parts[14].right(1) != "0");
+            QCOMPARE(tz.isDstAtUtc(utc), dst);
+            QCOMPARE(parts[15].mid(7).toInt(), tz.offsetAtUtc(utc));
+        }
+        file.close();
+    }
 }
 
 void KTimeZonesTest::tzfileToZoneTime()
@@ -474,7 +483,7 @@ void KTimeZonesTest::tzfileToZoneTime()
     // Convert from UTC.
     KTzfileTimeZoneSource tzsource(KSystemTimeZones::zoneinfoDir());
     KTimeZone london = KTzfileTimeZone(&tzsource, "Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     QCOMPARE(london.type(), QByteArray("KTzfileTimeZone"));
     QDateTime prepre(QDate(2005,10,29), QTime(23,59,59), Qt::UTC);  // before time shift (local time not repeated)
     QDateTime pre(QDate(2005,10,30), QTime(0,0,0), Qt::UTC);  // before time shift (local time repeated afterwards)
@@ -497,7 +506,7 @@ void KTimeZonesTest::tzfileToZoneTime()
     QVERIFY(!second);
 
     KTimeZone sysLondon = KSystemTimeZones::readZone("Europe/London");
-    QVERIFY( sysLondon.isValid() );
+    QVERIFY(sysLondon.isValid());
     QCOMPARE(sysLondon.toZoneTime(prepre, &second), QDateTime(QDate(2005,10,30), QTime(0,59,59), Qt::LocalTime));
     QVERIFY(!second);
     QCOMPARE(sysLondon.toZoneTime(pre, &second), QDateTime(QDate(2005,10,30), QTime(1,0,0), Qt::LocalTime));
@@ -512,52 +521,124 @@ void KTimeZonesTest::tzfileToZoneTime()
     QVERIFY(!second);
 }
 
-void KTimeZonesTest::tzfileOffsetAtZoneTime()
+void KTimeZonesTest::tzfileOffsetAtUtc()
 {
-    QDateTime aGmt(QDate(2005,3,27), QTime(0,30,0), Qt::LocalTime);
-    QDateTime aInvalid(QDate(2005,3,27), QTime(1,30,0), Qt::LocalTime);
-    QDateTime aBst(QDate(2005,3,27), QTime(2,30,0), Qt::LocalTime);
-    QDateTime bBst(QDate(2005,10,30), QTime(0,30,0), Qt::LocalTime);
-    QDateTime bBstBeforeGmt(QDate(2005,10,30), QTime(1,30,0), Qt::LocalTime);
-    QDateTime bGmt(QDate(2005,10,30), QTime(2,30,0), Qt::LocalTime);
+    QDateTime a3Gmt(QDate(2004,12,27), QTime(0,30,0), Qt::UTC);
+    QDateTime a2Gmt(QDate(2005,2,27), QTime(0,30,0), Qt::UTC);
+    QDateTime aGmt(QDate(2005,3,27), QTime(0,30,0), Qt::UTC);
+    QDateTime aBst(QDate(2005,3,27), QTime(1,30,0), Qt::UTC);
+    QDateTime a2Bst(QDate(2005,7,27), QTime(2,30,0), Qt::UTC);
+    QDateTime bBst(QDate(2005,10,29), QTime(23,30,0), Qt::UTC);
+    QDateTime bBstBeforeGmt(QDate(2005,10,30), QTime(0,30,0), Qt::UTC);
+    QDateTime bGmt(QDate(2005,10,30), QTime(2,30,0), Qt::UTC);
     KTzfileTimeZoneSource tzsource(KSystemTimeZones::zoneinfoDir());
     KTimeZone london = KTzfileTimeZone(&tzsource, "Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
+    QCOMPARE(london.type(), QByteArray("KTzfileTimeZone"));
+    QCOMPARE(london.offsetAtUtc(a3Gmt), 0);
+    QCOMPARE(london.offsetAtUtc(a2Gmt), 0);             // uses cache
+    QCOMPARE(london.offsetAtUtc(aGmt), 0);              // uses cache
+    QCOMPARE(london.offsetAtUtc(aBst), 3600);
+    QCOMPARE(london.offsetAtUtc(a2Bst), 3600);          // uses cache
+    QCOMPARE(london.offsetAtUtc(bBst), 3600);           // uses cache
+    QCOMPARE(london.offsetAtUtc(bBstBeforeGmt), 3600);  // uses cache
+    QCOMPARE(london.offsetAtUtc(bGmt), 0);
+
+    QDateTime recent(QDate(2013,5,10), QTime(13,0,0), Qt::UTC);
+    KTimeZone johannesburg = KTzfileTimeZone(&tzsource, "Africa/Johannesburg");
+    QVERIFY(johannesburg.isValid());
+    QCOMPARE(johannesburg.type(), QByteArray("KTzfileTimeZone"));
+    QCOMPARE(johannesburg.offsetAtUtc(recent), 7200);
+}
+
+void KTimeZonesTest::tzfileOffsetAtZoneTime()
+{
+    QDateTime Gmt0(QDate(2005,2,27), QTime(0,30,0), Qt::LocalTime);
+    QDateTime Gmt1(QDate(2005,3,27), QTime(0,30,0), Qt::LocalTime);
+    QDateTime GmtToBst1(QDate(2005,3,27), QTime(0,59,59), Qt::LocalTime);
+    QDateTime GmtToBst2(QDate(2005,3,27), QTime(1,0,0), Qt::LocalTime);
+    QDateTime GmtToBst3(QDate(2005,3,27), QTime(1,59,59), Qt::LocalTime);
+    QDateTime GmtToBst4(QDate(2005,3,27), QTime(2,0,0), Qt::LocalTime);
+    QDateTime Bst1(QDate(2005,4,27), QTime(2,30,0), Qt::LocalTime);
+    QDateTime Bst2(QDate(2005,10,30), QTime(0,30,0), Qt::LocalTime);
+    QDateTime BstToGmt1(QDate(2005,10,30), QTime(0,59,59), Qt::LocalTime);
+    QDateTime BstToGmt2(QDate(2005,10,30), QTime(1,0,0), Qt::LocalTime);
+    QDateTime BstToGmt3(QDate(2005,10,30), QTime(1,59,59), Qt::LocalTime);
+    QDateTime BstToGmt4(QDate(2005,10,30), QTime(2,0,0), Qt::LocalTime);
+    QDateTime Gmt2(QDate(2005,10,30), QTime(2,30,0), Qt::LocalTime);
+    KTzfileTimeZoneSource tzsource(KSystemTimeZones::zoneinfoDir());
+    KTimeZone london = KTzfileTimeZone(&tzsource, "Europe/London");
+    QVERIFY(london.isValid());
     int offset2;
-    QCOMPARE(london.offsetAtZoneTime(aGmt, &offset2), 0);
+    QCOMPARE(london.offsetAtZoneTime(Gmt0, &offset2), 0);
     QCOMPARE(offset2, 0);
-    QCOMPARE(london.offsetAtZoneTime(aInvalid, &offset2), KTimeZone::InvalidOffset);
+    QCOMPARE(london.offsetAtZoneTime(Gmt1, &offset2), 0);          // uses cache
+    QCOMPARE(offset2, 0);
+    QCOMPARE(london.offsetAtZoneTime(GmtToBst1, &offset2), 0);     // uses cache
+    QCOMPARE(offset2, 0);
+    QCOMPARE(london.offsetAtZoneTime(GmtToBst2, &offset2), KTimeZone::InvalidOffset);
     QCOMPARE(offset2, KTimeZone::InvalidOffset);
-    QCOMPARE(london.offsetAtZoneTime(aBst, &offset2), 3600);
+    QCOMPARE(london.offsetAtZoneTime(GmtToBst3, &offset2), KTimeZone::InvalidOffset);
+    QCOMPARE(offset2, KTimeZone::InvalidOffset);
+    QCOMPARE(london.offsetAtZoneTime(GmtToBst4, &offset2), 3600);
     QCOMPARE(offset2, 3600);
-    QCOMPARE(london.offsetAtZoneTime(bBst, &offset2), 3600);
+    QCOMPARE(london.offsetAtZoneTime(Bst1, &offset2), 3600);
     QCOMPARE(offset2, 3600);
-    QCOMPARE(london.offsetAtZoneTime(bBstBeforeGmt, &offset2), 3600);
+    QCOMPARE(london.offsetAtZoneTime(Bst2, &offset2), 3600);       // uses cache
+    QCOMPARE(offset2, 3600);
+    QCOMPARE(london.offsetAtZoneTime(BstToGmt1, &offset2), 3600);  // uses cache
+    QCOMPARE(offset2, 3600);
+    QCOMPARE(london.offsetAtZoneTime(BstToGmt2, &offset2), 3600);
     QCOMPARE(offset2, 0);
-    QCOMPARE(london.offsetAtZoneTime(bGmt, &offset2), 0);
+    QCOMPARE(london.offsetAtZoneTime(BstToGmt3, &offset2), 3600);
+    QCOMPARE(offset2, 0);
+    QCOMPARE(london.offsetAtZoneTime(BstToGmt4, &offset2), 0);
+    QCOMPARE(offset2, 0);
+    QCOMPARE(london.offsetAtZoneTime(Gmt2, &offset2), 0);          // uses cache
     QCOMPARE(offset2, 0);
 
     KTimeZone sysLondon = KSystemTimeZones::readZone("Europe/London");
-    QVERIFY( sysLondon.isValid() );
-    QCOMPARE(sysLondon.offsetAtZoneTime(aGmt, &offset2), 0);
+    QVERIFY(sysLondon.isValid());
+    QCOMPARE(sysLondon.offsetAtZoneTime(Gmt0, &offset2), 0);
     QCOMPARE(offset2, 0);
-    QCOMPARE(sysLondon.offsetAtZoneTime(aInvalid, &offset2), KTimeZone::InvalidOffset);
+    QCOMPARE(sysLondon.offsetAtZoneTime(Gmt1, &offset2), 0);          // uses cache
+    QCOMPARE(offset2, 0);
+    QCOMPARE(sysLondon.offsetAtZoneTime(GmtToBst1, &offset2), 0);     // uses cache
+    QCOMPARE(offset2, 0);
+    QCOMPARE(sysLondon.offsetAtZoneTime(GmtToBst2, &offset2), KTimeZone::InvalidOffset);
     QCOMPARE(offset2, KTimeZone::InvalidOffset);
-    QCOMPARE(sysLondon.offsetAtZoneTime(aBst, &offset2), 3600);
+    QCOMPARE(sysLondon.offsetAtZoneTime(GmtToBst3, &offset2), KTimeZone::InvalidOffset);
+    QCOMPARE(offset2, KTimeZone::InvalidOffset);
+    QCOMPARE(sysLondon.offsetAtZoneTime(GmtToBst4, &offset2), 3600);
     QCOMPARE(offset2, 3600);
-    QCOMPARE(sysLondon.offsetAtZoneTime(bBst, &offset2), 3600);
+    QCOMPARE(sysLondon.offsetAtZoneTime(Bst1, &offset2), 3600);
     QCOMPARE(offset2, 3600);
-    QCOMPARE(sysLondon.offsetAtZoneTime(bBstBeforeGmt, &offset2), 3600);
+    QCOMPARE(sysLondon.offsetAtZoneTime(Bst2, &offset2), 3600);       // uses cache
+    QCOMPARE(offset2, 3600);
+    QCOMPARE(sysLondon.offsetAtZoneTime(BstToGmt1, &offset2), 3600);  // uses cache
+    QCOMPARE(offset2, 3600);
+    QCOMPARE(sysLondon.offsetAtZoneTime(BstToGmt2, &offset2), 3600);
     QCOMPARE(offset2, 0);
-    QCOMPARE(sysLondon.offsetAtZoneTime(bGmt, &offset2), 0);
+    QCOMPARE(sysLondon.offsetAtZoneTime(BstToGmt3, &offset2), 3600);
     QCOMPARE(offset2, 0);
+    QCOMPARE(sysLondon.offsetAtZoneTime(BstToGmt4, &offset2), 0);
+    QCOMPARE(offset2, 0);
+    QCOMPARE(sysLondon.offsetAtZoneTime(Gmt2, &offset2), 0);          // uses cache
+    QCOMPARE(offset2, 0);
+
+    KTimeZone johannesburg = KTzfileTimeZone(&tzsource, "Africa/Johannesburg");
+    QVERIFY(johannesburg.isValid());
+    QCOMPARE(johannesburg.type(), QByteArray("KTzfileTimeZone"));
+    QDateTime recent(QDate(2013,5,10), QTime(13,0,0), Qt::LocalTime);
+    QCOMPARE(johannesburg.offsetAtZoneTime(recent, &offset2), 7200);
+    QCOMPARE(offset2, 7200);
 }
 
 void KTimeZonesTest::tzfileUtcOffsets()
 {
     KTzfileTimeZoneSource tzsource(KSystemTimeZones::zoneinfoDir());
     KTimeZone london = KTzfileTimeZone(&tzsource, "Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     QList<int> offsets = london.utcOffsets();
     QCOMPARE(offsets.count(), 3);
     QCOMPARE(offsets[0], 0);    // GMT
@@ -565,7 +646,7 @@ void KTimeZonesTest::tzfileUtcOffsets()
     QCOMPARE(offsets[2], 7200); // DST
 
     KTimeZone sysLondon = KSystemTimeZones::readZone("Europe/London");
-    QVERIFY( sysLondon.isValid() );
+    QVERIFY(sysLondon.isValid());
     offsets = sysLondon.utcOffsets();
     QCOMPARE(offsets.count(), 3);
     QCOMPARE(offsets[0], 0);    // GMT
@@ -577,7 +658,7 @@ void KTimeZonesTest::tzfileAbbreviation()
 {
     KTzfileTimeZoneSource tzsource(KSystemTimeZones::zoneinfoDir());
     KTimeZone london = KTzfileTimeZone(&tzsource, "Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     QDateTime winter(QDate(2005,1,1), QTime(0,0,0), Qt::UTC);
     QDateTime summer(QDate(2005,6,1), QTime(0,0,0), Qt::UTC);
     QDateTime standard(QDate(1970,4,30), QTime(12,45,16,25), Qt::UTC);
@@ -587,7 +668,7 @@ void KTimeZonesTest::tzfileAbbreviation()
     QCOMPARE(london.abbreviation(standard), QByteArray("BST"));
 
     KTimeZone sysLondon = KSystemTimeZones::readZone("Europe/London");
-    QVERIFY( sysLondon.isValid() );
+    QVERIFY(sysLondon.isValid());
     sResult = sysLondon.abbreviation(winter);
     QCOMPARE(sysLondon.abbreviation(winter), QByteArray("GMT"));
     QCOMPARE(sysLondon.abbreviation(summer), QByteArray("BST"));
@@ -598,7 +679,7 @@ void KTimeZonesTest::tzfileTransitions()
 {
     KTzfileTimeZoneSource tzsource(KSystemTimeZones::zoneinfoDir());
     KTimeZone london = KTzfileTimeZone(&tzsource, "Europe/London");
-    QVERIFY( london.isValid() );
+    QVERIFY(london.isValid());
     QList<KTimeZone::Transition> all = london.transitions();
     QVERIFY(!all.isEmpty());
     QDateTime jan2003(QDate(2003,1,1),QTime(0,0,0),Qt::UTC);
@@ -643,3 +724,4 @@ void KTimeZonesTest::tzfileTransitions()
         }
     }
 }
+
