@@ -48,10 +48,6 @@ static const int computedProperties[] = {
     CSS_PROP_BORDER_SPACING,
     CSS_PROP__KHTML_BORDER_HORIZONTAL_SPACING,
     CSS_PROP__KHTML_BORDER_VERTICAL_SPACING,
-    CSS_PROP__KHTML_BORDER_TOP_RIGHT_RADIUS,
-    CSS_PROP__KHTML_BORDER_BOTTOM_RIGHT_RADIUS,
-    CSS_PROP__KHTML_BORDER_BOTTOM_LEFT_RADIUS,
-    CSS_PROP__KHTML_BORDER_TOP_LEFT_RADIUS,
     CSS_PROP_BORDER_TOP_RIGHT_RADIUS,
     CSS_PROP_BORDER_BOTTOM_RIGHT_RADIUS,
     CSS_PROP_BORDER_BOTTOM_LEFT_RADIUS,
@@ -138,7 +134,7 @@ static const int computedProperties[] = {
 const unsigned numComputedProperties = sizeof(computedProperties) / sizeof(computedProperties[0]);
 
 
-static CSSValueImpl *valueForLength(const Length &length, int max)
+static CSSPrimitiveValueImpl *valueForLength(const Length &length, int max)
 {
     if (length.isPercent()) {
         return new CSSPrimitiveValueImpl(length.percent(), CSSPrimitiveValue::CSS_PERCENTAGE);
@@ -148,7 +144,7 @@ static CSSValueImpl *valueForLength(const Length &length, int max)
     }
 }
 
-static CSSValueImpl *valueForLength2(const Length &length)
+static CSSPrimitiveValueImpl *valueForLength2(const Length &length)
 {
     if (length.isPercent()) {
         return new CSSPrimitiveValueImpl(length.percent(), CSSPrimitiveValue::CSS_PERCENTAGE);
@@ -190,8 +186,8 @@ static CSSValueImpl *valueForBorderStyle(EBorderStyle style)
 
 static CSSValueImpl *valueForBorderRadii(BorderRadii radii)
 {
-    CSSPrimitiveValueImpl* h = new CSSPrimitiveValueImpl(radii.horizontal, CSSPrimitiveValue::CSS_PX);
-    CSSPrimitiveValueImpl* v = new CSSPrimitiveValueImpl(radii.vertical, CSSPrimitiveValue::CSS_PX);
+    CSSPrimitiveValueImpl* h = valueForLength2(radii.horizontal);
+    CSSPrimitiveValueImpl* v = valueForLength2(radii.vertical);
     return new CSSPrimitiveValueImpl(new PairImpl(h, v));
 }
 
@@ -538,16 +534,12 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
         return new CSSPrimitiveValueImpl(style->borderHorizontalSpacing(), CSSPrimitiveValue::CSS_PX);
     case CSS_PROP__KHTML_BORDER_VERTICAL_SPACING:
         return new CSSPrimitiveValueImpl(style->borderVerticalSpacing(), CSSPrimitiveValue::CSS_PX);
-    case CSS_PROP__KHTML_BORDER_TOP_RIGHT_RADIUS:
     case CSS_PROP_BORDER_TOP_RIGHT_RADIUS:
 	return valueForBorderRadii(style->borderTopRightRadius());
-    case CSS_PROP__KHTML_BORDER_BOTTOM_RIGHT_RADIUS:
     case CSS_PROP_BORDER_BOTTOM_RIGHT_RADIUS:
 	return valueForBorderRadii(style->borderBottomRightRadius());
-    case CSS_PROP__KHTML_BORDER_BOTTOM_LEFT_RADIUS:
     case CSS_PROP_BORDER_BOTTOM_LEFT_RADIUS:
 	return valueForBorderRadii(style->borderBottomLeftRadius());
-    case CSS_PROP__KHTML_BORDER_TOP_LEFT_RADIUS:
     case CSS_PROP_BORDER_TOP_LEFT_RADIUS:
 	return valueForBorderRadii(style->borderTopLeftRadius());
     case CSS_PROP_BORDER_TOP_COLOR:
