@@ -55,7 +55,7 @@ void HTMLBaseElementImpl::parseAttribute(AttributeImpl *attr)
     switch(attr->id())
     {
     case ATTR_HREF:
-	m_href = attr->value().trimSpaces().string();
+	m_href = attr->value().parsedUrl().string();
 	process();
 	break;
     case ATTR_TARGET:
@@ -116,7 +116,7 @@ void HTMLLinkElementImpl::parseAttribute(AttributeImpl *attr)
     switch (attr->id())
     {
     case ATTR_HREF: {
-        const DOMString hrefUrl = attr->value().trimSpaces();
+        const DOMString hrefUrl = attr->value().parsedUrl();
         if (!hrefUrl.isEmpty()) {
             m_url = document()->completeURL(hrefUrl.string());
         }
@@ -173,19 +173,37 @@ void HTMLLinkElementImpl::parseAttribute(AttributeImpl *attr)
     }
 }
 
+static inline bool isIconLink(const QString &relAttribute)
+{
+    // http://www.w3.org/TR/html5/links.html#linkTypes:
+    // split rel attribute on spaces and search for "icon" keyword.
+    // note: relAttribute is supposed to be a lower case string
+    const QLatin1String icon("icon");
+    int iconPos = relAttribute.indexOf(icon);
+    while (iconPos != -1) {
+        const bool found = (iconPos == 0 || relAttribute.at(iconPos - 1).isSpace()) &&
+                           (((iconPos + 4) == relAttribute.length()) || relAttribute.at(iconPos + 4).isSpace());
+        if (found) {
+            return true;
+        }
+        iconPos = relAttribute.indexOf(icon, iconPos + 5);
+    }
+
+    return false;
+}
+
 void HTMLLinkElementImpl::process()
 {
     if (!inDocument())
         return;
 
-    QString type = getAttribute(ATTR_TYPE).string().toLower();
-    QString rel = getAttribute(ATTR_REL).string().toLower();
+    //QString type = getAttribute(ATTR_TYPE).string().toLower();
+    const QString rel = getAttribute(ATTR_REL).string().toLower();
 
     KHTMLPart* part = document()->part();
 
-    // IE extension: location of small icon for locationbar / bookmarks
-    // Uses both "shortcut icon" and "icon"
-    if (rel.contains("icon") && !m_url.isEmpty() && part && !part->parentPart()) {
+    // Location of small icon for locationbar / bookmarks (aka "favicon")
+    if (isIconLink(rel) && !m_url.isEmpty() && part && !part->parentPart()) {
         part->browserExtension()->setIconUrl( KUrl(m_url.string()) );
     } // Stylesheet
     else if (rel.contains("stylesheet") && !m_url.isEmpty() && !m_isDisabled) {
@@ -385,7 +403,7 @@ void HTMLScriptElementImpl::parseAttribute(AttributeImpl *attr)
         // we'll only start doing things once we get insertedIntoDocument()
         if (m_evaluated || m_cachedScript || m_createdByParser || !inDocument())
             return;
-        const DOMString url = attr->value().trimSpaces();
+        const DOMString url = attr->value().parsedUrl();
         if (!url.isEmpty())
             loadFromUrl(url);
         break;
@@ -490,7 +508,7 @@ void HTMLScriptElementImpl::insertedIntoDocument()
     if (m_createdByParser)
         return;
 
-    const DOMString url = getAttribute(ATTR_SRC).trimSpaces();
+    const DOMString url = getAttribute(ATTR_SRC).parsedUrl();
     if (!url.isEmpty()) {
         loadFromUrl(url);
         return;
@@ -622,7 +640,7 @@ void HTMLScriptElementImpl::setDefer(bool defer)
 
 DOMString HTMLScriptElementImpl::src() const
 {
-    return document()->completeURL(getAttribute(ATTR_SRC).trimSpaces().string());
+    return document()->completeURL(getAttribute(ATTR_SRC).parsedUrl().string());
 }
 
 void HTMLScriptElementImpl::setSrc(const DOMString &value)
