@@ -26,18 +26,15 @@
 #include <QGraphicsProxyWidget>
 
 #include <kactioncollection.h>
+#include <kconfigdialog.h>
 
 #include "plasma/animator.h"
+#include "plasma/private/applethandle_p.h"
 #include "plasma/private/dataengineconsumer_p.h"
 #include "plasma/ui_publish.h"
 
 class KKeySequenceWidget;
 
-namespace KIO
-{
-    class Job;
-}
-
 namespace Plasma
 {
 
@@ -46,6 +43,35 @@ class FrameSvg;
 class AppletScript;
 class Wallpaper;
 class BusyWidget;
+class PushButton;
+
+class AppletConfigDialog : public KConfigDialog
+{
+    Q_OBJECT
+
+public:
+    AppletConfigDialog(QWidget* parent, const QString &id, KConfigSkeleton *s)
+        : KConfigDialog(parent, id, s),
+          m_changed(false)
+    {
+    }
+
+public Q_SLOTS:
+    void settingsModified(bool modified = true)
+    {
+        m_changed = modified;
+        updateButtons();
+    }
+
+protected:
+    bool hasChanged()
+    {
+        return m_changed || KConfigDialog::hasChanged();
+    }
+
+private:
+    bool m_changed;
+};
 
 class AppletOverlayWidget : public QGraphicsWidget
 {
@@ -59,6 +85,7 @@ public:
 
 protected:
     void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
+    void mousePressEvent(QGraphicsSceneMouseEvent *event);
 
 protected Q_SLOTS:
     void overlayAnimationComplete();
@@ -67,7 +94,7 @@ protected Q_SLOTS:
 class AppletPrivate : public DataEngineConsumer
 {
 public:
-    AppletPrivate(KService::Ptr service, int uniqueID, Applet *applet);
+    AppletPrivate(KService::Ptr service, const KPluginInfo *info, int uniqueID, Applet *applet);
     ~AppletPrivate();
 
     void init(const QString &packagePath = QString());
@@ -81,16 +108,15 @@ public:
      */
     void setIsContainment(bool isContainment, bool forceUpdate = false);
 
+    void handleDisappeared(AppletHandle *handle);
     QString globalName() const;
     QString instanceName();
     void scheduleConstraintsUpdate(Plasma::Constraints c);
     void scheduleModificationNotification();
     KConfigGroup *mainConfigGroup();
     QString visibleFailureText(const QString &reason);
-    void checkImmutability();
     void themeChanged();
     void resetConfigurationObject();
-    void appletAnimationComplete();
     void selectItemToDestroy();
     void updateRect(const QRectF &rect);
     void setFocus();
@@ -99,8 +125,6 @@ public:
     void positionMessageOverlay();
     void destroyMessageOverlay();
     void addGlobalShortcutsPage(KConfigDialog *dialog);
-    void addPublishPage(KConfigDialog *dialog);
-    void clearShortcutEditorPtr();
     void configDialogFinished();
     KConfigDialog *generateGenericConfigDialog();
     void addStandardConfigurationPages(KConfigDialog *dialog);
@@ -108,10 +132,13 @@ public:
     QString configWindowTitle() const;
     void updateShortcuts();
     void publishCheckboxStateChanged(int state);
+    void globalShortcutChanged();
+    void propagateConfigChanged();
 
     static KActionCollection* defaultActions(QObject *parent);
     static QSet<QString> knownCategories();
     static void filterOffers(QList<KService::Ptr> &offers);
+    static QString parentAppConstraint(const QString &parentApp = QString());
 
     static uint s_maxAppletId;
     static int s_maxZValue;
@@ -147,23 +174,35 @@ public:
     AppletOverlayWidget *messageOverlay;
     QGraphicsProxyWidget *messageOverlayProxy;
     Plasma::BusyWidget *busyWidget;
+    QWeakPointer<Plasma::PushButton> messageOkButton;
+    QWeakPointer<Plasma::PushButton> messageYesButton;
+    QWeakPointer<Plasma::PushButton> messageNoButton;
+    QWeakPointer<Plasma::PushButton> messageCancelButton;
+    QWeakPointer<QAction> messageCloseAction;
 
     // sripting and package stuff
     AppletScript *script;
     Package *package;
     ConfigLoader *configLoader;
+    QWeakPointer<AppletHandle> handle;
 
     // actions stuff; put activationAction into actions?
     KActionCollection *actions;
     KAction *activationAction;
 
     // configuration
-    KKeySequenceWidget *shortcutEditor; //TODO: subclass KConfigDialog and encapsulate this in there
+    QWeakPointer<KKeySequenceWidget> shortcutEditor; //TODO: subclass KConfigDialog and encapsulate this in there
 
     ItemStatus itemStatus;
     QString remoteLocation;
     Ui::publishWidget publishUI;
 
+    // the applet can change size policy by itself, so save the old one for eventual restore
+    QSizePolicy preferredSizePolicy;
+
+    //keep last sizes for formfactors, useful when the containment changes
+    QHash<FormFactor, QSizeF> sizeForFormFactor;
+
     // timerEvent bookkeeping
     QBasicTimer constraintsTimer;
     QBasicTimer busyWidgetTimer;
